import 'dart:async';

import 'package:nexstate/nexstate.dart';
import 'package:test/test.dart';

void main() {
  test('Manages State Correctly', () async {
    final userStore = UserStore();
    final subscription = userStore.runAndSubscribe(() => print(userStore.user));

    const johnDoe = User(
      firstName: 'John',
      lastName: 'Doe',
      userName: 'john_doe',
    );

    const janeDoe = User(
      firstName: 'Jane',
      lastName: 'Doe',
      userName: 'jane_doe',
    );

    userStore.login(johnDoe);
    expect(userStore.user, johnDoe);

    await userStore.switchAndLogout(janeDoe);
    expect(userStore.user, null);

    Timer.run(subscription.cancel);
  });
}

class User {
  final String firstName;
  final String lastName;
  final String userName;

  const User({
    required this.firstName,
    required this.lastName,
    required this.userName,
  });

  @override
  String toString() {
    return {
      'firstName': firstName,
      'lastName': lastName,
      'userName': userName,
    }.toString();
  }
}

class UserStore extends Store {
  User? user;

  void login(User newUser) => setState(() => user = newUser);
  void logout() => setState(() => user = null);

  Future<void> switchAndLogout(User user) async {
    login(user);
    await Future.delayed(const Duration(seconds: 2), logout);
  }
}
